@@ -102,12 +102,6 @@
 /* Libraries */
 /* #undef HAVE_LIBWRAP */
 /* #undef HAVE_SYSTEMD */
-/* #undef HAVE_CRC32_VPMSUM */
-
-/* Support ARMv8 crc + crypto  */
-/* #undef HAVE_ARMV8_CRC */
-/* #undef HAVE_ARMV8_CRYPTO */
-/* #undef HAVE_ARMV8_CRC_CRYPTO_INTRINSICS */
 
 /* Does "struct timespec" have a "sec" and "nsec" field? */
 /* #undef HAVE_TIMESPEC_TS_SEC */
@@ -395,8 +389,8 @@
 #define STACK_DIRECTION -1
 
 #define SYSTEM_TYPE "OpenBSD"
-#define MACHINE_TYPE "amd64"
-#define DEFAULT_MACHINE "amd64"
+#define MACHINE_TYPE "aarch64"
+#define DEFAULT_MACHINE "aarch64"
 /* #undef HAVE_DTRACE */
 
 #define SIGNAL_WITH_VIO_CLOSE 1
@@ -531,16 +525,16 @@
 
 #define MYSQL_VERSION_MAJOR 10
 #define MYSQL_VERSION_MINOR 5
-#define MYSQL_VERSION_PATCH 6
+#define MYSQL_VERSION_PATCH 8
 #define MYSQL_VERSION_EXTRA ""
 
 #define PACKAGE "mysql"
 #define PACKAGE_BUGREPORT ""
 #define PACKAGE_NAME "MySQL Server"
-#define PACKAGE_STRING "MySQL Server 10.5.6"
+#define PACKAGE_STRING "MySQL Server 10.5.8"
 #define PACKAGE_TARNAME "mysql"
-#define PACKAGE_VERSION "10.5.6"
-#define VERSION "10.5.6"
+#define PACKAGE_VERSION "10.5.8"
+#define VERSION "10.5.8"
 #define PROTOCOL_VERSION 10
 #define PCRE2_CODE_UNIT_WIDTH 8
 
